/**
 * <a href="http://www.openolat.org">
 * OpenOLAT - Online Learning and Training</a><br>
 * <p>
 * Licensed under the Apache License, Version 2.0 (the "License"); <br>
 * you may not use this file except in compliance with the License.<br>
 * You may obtain a copy of the License at the
 * <a href="http://www.apache.org/licenses/LICENSE-2.0">Apache homepage</a>
 * <p>
 * Unless required by applicable law or agreed to in writing,<br>
 * software distributed under the License is distributed on an "AS IS" BASIS, <br>
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. <br>
 * See the License for the specific language governing permissions and <br>
 * limitations under the License.
 * <p>
 * Initial code contributed and copyrighted by<br>
 * frentix GmbH, http://www.frentix.com
 * <p>
 */
package org.olat.core.util;

import static org.junit.Assert.assertFalse;
import static org.junit.Assert.assertTrue;

import org.junit.Assert;
import org.junit.Test;

/**
 * Description:<br>
 * This test case tests the StringHelper methods
 * 
 * <P>
 * Initial Date:  13.07.2010 <br>
 * @author gnaegi
 */
public class StringHelperTest {

	@Test
	public void testContainsNonWhitespace() {
		// positive tests
		assertTrue(StringHelper.containsNonWhitespace("asdf"));
		assertTrue(StringHelper.containsNonWhitespace("  asdf"));
		assertTrue(StringHelper.containsNonWhitespace("asdf  "));
		assertTrue(StringHelper.containsNonWhitespace("asdf  t\r"));
		assertTrue(StringHelper.containsNonWhitespace("hello world"));
		// negative tests
		assertFalse(StringHelper.containsNonWhitespace(null));
		assertFalse(StringHelper.containsNonWhitespace(""));
		assertFalse(StringHelper.containsNonWhitespace(" "));
		assertFalse(StringHelper.containsNonWhitespace("             "));
		assertFalse(StringHelper.containsNonWhitespace("  \t  \r"));
	}
	
	@Test
	public void transformDisplayNameToFileSystemName() {
		Assert.assertEquals("Webclass_Energie_2004_2005", StringHelper.transformDisplayNameToFileSystemName("Webclass Energie 2004/2005"));
		Assert.assertEquals("Webclass_Energie_2004_2005", StringHelper.transformDisplayNameToFileSystemName("Webclass Energie 2004\\2005"));
		Assert.assertEquals("Webclass_Energie_20042005", StringHelper.transformDisplayNameToFileSystemName("Webclass Energie 2004:2005"));
		Assert.assertEquals("Webclaess", StringHelper.transformDisplayNameToFileSystemName("Webcl\u00E4ss"));
	}
	
	@Test
	public void filterPrintControlCharacter() {
		String value1 = StringHelper.cleanUTF8ForXml("Hello world");
		Assert.assertEquals("Dummy test", "Hello world", value1);
		
		//print control
		String value2 = StringHelper.cleanUTF8ForXml("Hello\u0002 world");
		Assert.assertEquals("Print \\x02 test", "Hello world", value2);

		//print control
		String value3 = StringHelper.cleanUTF8ForXml("Hello\u001F world");
		Assert.assertEquals("Print \\x02 like test", "Hello world", value3);

		//it's a 0
		String value4 = StringHelper.cleanUTF8ForXml("Hello\u0030 world");
		Assert.assertEquals("Zero test", "Hello0 world", value4);
			
		//it's a u umlaut
		String value5 = StringHelper.cleanUTF8ForXml("Hello\u00FC world");
		Assert.assertEquals("Umlaut test", "Hello\u00FC world", value5);
					
		//it's a kanji
		String value6 = StringHelper.cleanUTF8ForXml("Hello\u30b0 world");
		Assert.assertEquals("Kanji test", "Hello\u30b0 world", value6);
		
		//it's a return
		String value7 = StringHelper.cleanUTF8ForXml("Hello\n world");
		Assert.assertEquals("Return test", "Hello\n world", value7);
		
		//it's a tab
		String value8 = StringHelper.cleanUTF8ForXml("Hello\t world");
		Assert.assertEquals("Tab test", "Hello\t world", value8);
		
		//it's a carriage
		String value9 = StringHelper.cleanUTF8ForXml("Hello\r world");
		Assert.assertEquals("Carriage test", "Hello\r world", value9);
		
		//it's a unicode emoticons
		String value10 = StringHelper.cleanUTF8ForXml("Hello\u1F605 world");
		Assert.assertEquals("Emoticons test", "Hello\u1F605 world", value10);
		
		//it's phoenician \u1090x
		String value11 = StringHelper.cleanUTF8ForXml("Hello\u1090x phoenician");
		Assert.assertEquals("Phoenician test", "Hello\u1090x phoenician", value11);
		
		//it's pahlavi \u10B7x
		String value12 = StringHelper.cleanUTF8ForXml("Hello\u10B7x pahlavi");
		Assert.assertEquals("Pahlavi test", "Hello\u10B7x pahlavi", value12);
	}
}
